﻿package fr.blueyeti
/**
 * MaxCom class : communication with Max  <br /><br />
 * 
 * <b>Copyright 2011, Blue Yeti </b>
 * 
 * @author Jean-Michel Couturier, jmc@blueyeti.fr
 */	

{

	import flash.display.DisplayObject;
	import flash.display.MovieClip;
	import flash.display.Sprite;
    import flash.events.*;
    import flash.system.*;
	import flash.ui.Mouse;
    import flash.utils.Timer;
    import flash.events.TimerEvent;
	import flash.net.XMLSocket;
	import flash.sampler.NewObjectSample;

	public class MaxCom extends Sprite
	{
		private var m_parent : MovieClip;
		private var m_hostName : String;
		private var m_port : uint;
		private var m_socket : XMLSocket;
		private var m_socketRetryConnectDelay : Timer;
	
		public function MaxCom(parent : MovieClip, iHost : String = '127.0.0.1', iPort : int = 31337) : void 
		{
			m_parent = parent;
			
			//Set attributes for connection with server
			m_hostName = iHost;
			m_port = iPort;
			
			//Create and set listeners for the xmlsocket
			m_socket = new XMLSocket();
			configureListeners(m_socket);
			
			//Connect to Max server
			m_socket.connect(m_hostName, m_port);
			
			//Set timer to delay connection trials and then maintain connection
			m_socketRetryConnectDelay=new Timer(10000,1);
			m_socketRetryConnectDelay.addEventListener(TimerEvent.TIMER_COMPLETE, tryConnect);
			m_socketRetryConnectDelay.start();
		}
		
		
		private function configureListeners(dispatcher : IEventDispatcher) : void 
		{
			dispatcher.addEventListener(Event.CLOSE, closeHandler);
			dispatcher.addEventListener(Event.CONNECT, connectHandler);
			dispatcher.addEventListener(DataEvent.DATA, dataHandler);
			dispatcher.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
			dispatcher.addEventListener(ProgressEvent.PROGRESS, progressHandler);
			dispatcher.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
		}
		
		
		private function tryConnect(evt : TimerEvent) : void 
		{
			//Connect 
			if (m_socket.connected == false)
			{
				m_socket.connect(m_hostName, m_port);
			}
			m_socketRetryConnectDelay.start();
		}


		private function closeHandler(evt : Event) : void 
		{
			trace("closeHandler: " + evt);
		}

		
		private function connectHandler(evt : Event) : void 
		{
			trace("connectHandler: "+evt+"\n");
			dispatchEvent(new Event('MaxConnected', true));
		}

		private function ioErrorHandler(evt : IOErrorEvent) : void 
		{
			trace("ioErrorHandler: " + evt + "\n");
			dispatchEvent(new Event('io_error', true));
		}

		
		private function progressHandler(evt : ProgressEvent) : void 
		{
			trace("progressHandler loaded:"+evt.bytesLoaded+" total: "+evt.bytesTotal+"\n");
		}

		
		private function securityErrorHandler(evt : SecurityErrorEvent) : void 
		{
			trace("securityErrorHandler: "+evt+"\n");
			dispatchEvent(new Event('security_error', true));
		}
		
		private function dataHandler(evt : DataEvent) : void 
		{	
			trace("dataHandler: " + evt + "\n");
			var message : String = evt.data.substr(0, evt.data.length - 1);
			m_parent.onMessageReceived(message);
		}
		
		public function play(value : int) : Boolean 
		{
			if (m_socket.connected == true) 
			{
				trace(String(value));
				m_socket.send("/play " + String(value) + ";");
				return true;
			}
			else
			{
				return false;
			}
		}
		
		public function stop() : Boolean 
		{
			if (m_socket.connected == true) 
			{
				m_socket.send("stop "+ ";");
				return true;
			}
			else
			{
				return false;
			}
		}
		
		public function send(message : String) : Boolean 
		{
			if (m_socket.connected == true) 
			{
				m_socket.send(message + ";");
				return true;
			}
			else
			{
				return false;
			}
		}
	}
}